"""Implementation of OpenGL errors/exceptions

Note that OpenGL-ctypes will also throw standard errors,
such as TypeError or ValueError when appropriate.

ErrorChecker is an _ErrorChecker instance that allows you
to register a new error-checking function for use 
throughout the system.
"""
import OpenGL
from OpenGL import platform
__all__ = (
	"Error",'GLError','GLUError','GLUTError','glCheckError',
	'GLerror','GLUerror','GLUTerror',
)

class Error( Exception ):
	"""Base class for all PyOpenGL-specific exception classes"""
class CopyError( Error ):
	"""Raised to indicate that operation requires data-copying
	
	Currently only supported by the numpy array formathandler,
	if you set:

		OpenGL.arrays.numpymodule.NumpyHandler.ERROR_ON_COPY = True 
	
	this error will be raised when the Numpy handler is passed
	a numpy array that requires copying to be passed to ctypes.
	"""

class NullFunctionError( Error ):
	"""Error raised when an undefined function is called"""

class GLError( Error ):
	"""OpenGL core error implementation class
	
	Primary purpose of this error class is to allow for 
	annotating an error with more details about the calling 
	environment so that it's easier to debug errors in the
	wrapping process.
	
	Attributes:
	
		err -- the OpenGL error code for the error 
		result -- the OpenGL result code for the operation
		baseOperation -- the "function" being called
		pyArgs -- the translated set of Python arguments
		cArgs -- the Python objects matching 1:1 the C arguments
		cArguments -- ctypes-level arguments to the operation,
			often raw integers for pointers and the like
		description -- OpenGL description of the error (textual)
	"""
	def __init__( 
		self, 
		err=None, 
		result=None, 
		cArguments=None, 
		baseOperation=None, 
		pyArgs=None, 
		cArgs=None,
		description=None,
	):
		"""Initialise the GLError, storing metadata for later display"""
		(
			self.err, self.result, self.cArguments, 
			self.baseOperation, self.pyArgs, self.cArgs,
			self.description
		) = (
			err, result, cArguments,
			baseOperation, pyArgs, cArgs,
			description
		)
	DISPLAY_ORDER = (
		'err', 
		'description',
		'baseOperation',
		'pyArgs', 
		'cArgs',
		'cArguments',
		'result', 
	)
	def __str__( self ):
		"""Create a fully formatted representation of the error"""
		args = []
		for property in self.DISPLAY_ORDER:
			value = getattr( self, property, None )
			if value is not None or property=='description':
				formatFunction = 'format_%s'%(property)
				if hasattr( self, formatFunction ):
					args.append( getattr(self,formatFunction)( property, value ))
				else:
					args.append( '%s = %s'%(
						property,
						self.shortRepr( value ),
					))
		return '%s(\n\t%s\n)'%(self.__class__.__name__, ',\n\t'.join(
			[x for x in args if x]
		))
	def __repr__( self ):
		"""Produce a much shorter version of the error as a string"""
		return '%s( %s )'%(
			self.__class__.__name__,
			", ".join([x for x in [
				'err=%s'%(self.err),
				self.format_description( 'description', self.description ) or '',
				self.format_baseOperation( 'baseOperation', self.baseOperation ) or '',
			] if x])
		)
	def format_description( self, property, value ):
		"""Format description using GLU's gluErrorString"""
		if value is None and self.err is not None:
			try:
				from OpenGL.GLU import gluErrorString
				self.description = value = gluErrorString( self.err )
			except Exception, err:
				return None
		if value is None:
			return None
		return '%s = %s'%(
			property,
			self.shortRepr( value ),
		)
	def shortRepr( self, value, firstLevel=True ):
		"""Retrieve short representation of the given value"""
		if isinstance( value, (list,tuple) ) and value and len(repr(value))>=40:
			if isinstance( value, list ):
				template = '[\n\t\t%s\n\t]'
			else:
				template = '(\n\t\t%s,\n\t)'
			return template%( ",\n\t\t".join(
				[
					self.shortRepr(x,False) for x in value
				]
			))
		r = repr( value )
		if len(r) < 40:
			return r
		else:
			return r[:37] + '...'
	def format_baseOperation( self, property, value ):
		"""Format a baseOperation reference for display"""
		if hasattr( value, '__name__' ):
			return '%s = %s'%( property, value.__name__ )
		else:
			return '%s = %r'%( property, value )

class GLUError( Error ):
	"""GLU error implementation class"""

class GLUTError( Error ):
	"""GLUT error implementation class"""


if OpenGL.ERROR_CHECKING:
	
	class _ErrorChecker( object ):
		"""Global error-checking object
		
		Attributes:
			_registeredChecker -- the checking function enabled when 
				not doing onBegin/onEnd processing
			safeGetError -- platform safeGetError function as callable method
			_currentChecker -- currently active checking function
		"""
		_currentChecker = _registeredChecker = safeGetError = staticmethod( 
			platform.safeGetError 
		)
		def glCheckError( 
			self,
			result,
			baseOperation=None,
			cArguments=None,
			*args
		):
			"""Base GL Error checker compatible with new ctypes errcheck protocol
			
			This function will raise a GLError with just the calling information
			available at the C-calling level, i.e. the error code, cArguments,
			baseOperation and result.  Higher-level code is responsible for any 
			extra annotations.
			
			Note:
				glCheckError relies on glBegin/glEnd interactions to 
				prevent glGetError being called during a glBegin/glEnd 
				sequence.  If you are calling glBegin/glEnd in C you 
				should call onBegin and onEnd appropriately.
			"""
			err = self._currentChecker()
			if err is not None and err != 0: # GL_NO_ERROR's guaranteed value
				raise GLError(
					err,
					result,
					cArguments = cArguments,
					baseOperation = baseOperation,
				)
			return result
		def nullGetError( self ):
			"""Used as error-checker when inside begin/end set"""
			return None
		def currentChecker( self ):
			"""Return the current error-checking function"""
			return self._currentChecker
		def registerChecker( self, checker=None ):
			"""Explicitly register an error checker
			
			This allows you to, for instance, disable error checking 
			entirely.  onBegin and onEnd will switch between 
			nullGetError and the value registered here.  If checker 
			is None then the default 
			"""
			if checker is None:
				checker = self.safeGetError
			self._registeredChecker = checker
			return checker
		def onBegin( self, target=None ):
			"""Called by glBegin to record the fact that glGetError won't work"""
			self._currentChecker = self.nullGetError
		def onEnd( self, target=None ):
			"""Called by glEnd to record the fact that glGetError will work"""
			self._currentChecker = self._registeredChecker
	
	ErrorChecker = _ErrorChecker()
	
	glCheckError = ErrorChecker.glCheckError
	onBegin = ErrorChecker.onBegin
	onEnd = ErrorChecker.onEnd
else:
	glCheckError = platform.safeGetError

# Compatibility with PyOpenGL 2.x series
GLUerror = GLUError
GLerror = GLError 
GLUTerror = GLUTError



'''OpenGL extension ATI.vertex_array_object

Overview (from the spec)
	
	This extension defines an interface that allows multiple sets of
	vertex array data to be cached in persistent server-side memory.
	It is intended to allow client data to be stored in memory that
	can be directly accessed by graphics hardware.
	

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/ATI/vertex_array_object.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ATI_vertex_array_object'
GL_STATIC_ATI = constant.Constant( 'GL_STATIC_ATI', 0x8760 )
GL_DYNAMIC_ATI = constant.Constant( 'GL_DYNAMIC_ATI', 0x8761 )
GL_PRESERVE_ATI = constant.Constant( 'GL_PRESERVE_ATI', 0x8762 )
GL_DISCARD_ATI = constant.Constant( 'GL_DISCARD_ATI', 0x8763 )
GL_OBJECT_BUFFER_SIZE_ATI = constant.Constant( 'GL_OBJECT_BUFFER_SIZE_ATI', 0x8764 )
GL_OBJECT_BUFFER_USAGE_ATI = constant.Constant( 'GL_OBJECT_BUFFER_USAGE_ATI', 0x8765 )
GL_ARRAY_OBJECT_BUFFER_ATI = constant.Constant( 'GL_ARRAY_OBJECT_BUFFER_ATI', 0x8766 )
GL_ARRAY_OBJECT_OFFSET_ATI = constant.Constant( 'GL_ARRAY_OBJECT_OFFSET_ATI', 0x8767 )
glNewObjectBufferATI = platform.createExtensionFunction( 
	'glNewObjectBufferATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=constants.GLuint, 
	argTypes=(constants.GLsizei, ctypes.c_void_p, constants.GLenum,),
	doc = 'glNewObjectBufferATI( GLsizei(size), c_void_p(pointer), GLenum(usage) ) -> constants.GLuint',
	argNames = ('size', 'pointer', 'usage',),
)

glIsObjectBufferATI = platform.createExtensionFunction( 
	'glIsObjectBufferATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=constants.GLboolean, 
	argTypes=(constants.GLuint,),
	doc = 'glIsObjectBufferATI( GLuint(buffer) ) -> constants.GLboolean',
	argNames = ('buffer',),
)

glUpdateObjectBufferATI = platform.createExtensionFunction( 
	'glUpdateObjectBufferATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLuint, constants.GLsizei, ctypes.c_void_p, constants.GLenum,),
	doc = 'glUpdateObjectBufferATI( GLuint(buffer), GLuint(offset), GLsizei(size), c_void_p(pointer), GLenum(preserve) ) -> None',
	argNames = ('buffer', 'offset', 'size', 'pointer', 'preserve',),
)

glGetObjectBufferfvATI = platform.createExtensionFunction( 
	'glGetObjectBufferfvATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, arrays.GLfloatArray,),
	doc = 'glGetObjectBufferfvATI( GLuint(buffer), GLenum(pname), GLfloatArray(params) ) -> None',
	argNames = ('buffer', 'pname', 'params',),
)

glGetObjectBufferivATI = platform.createExtensionFunction( 
	'glGetObjectBufferivATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, arrays.GLintArray,),
	doc = 'glGetObjectBufferivATI( GLuint(buffer), GLenum(pname), GLintArray(params) ) -> None',
	argNames = ('buffer', 'pname', 'params',),
)

glFreeObjectBufferATI = platform.createExtensionFunction( 
	'glFreeObjectBufferATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint,),
	doc = 'glFreeObjectBufferATI( GLuint(buffer) ) -> None',
	argNames = ('buffer',),
)

glArrayObjectATI = platform.createExtensionFunction( 
	'glArrayObjectATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLint, constants.GLenum, constants.GLsizei, constants.GLuint, constants.GLuint,),
	doc = 'glArrayObjectATI( GLenum(array), GLint(size), GLenum(type), GLsizei(stride), GLuint(buffer), GLuint(offset) ) -> None',
	argNames = ('array', 'size', 'type', 'stride', 'buffer', 'offset',),
)

glGetArrayObjectfvATI = platform.createExtensionFunction( 
	'glGetArrayObjectfvATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLenum, arrays.GLfloatArray,),
	doc = 'glGetArrayObjectfvATI( GLenum(array), GLenum(pname), GLfloatArray(params) ) -> None',
	argNames = ('array', 'pname', 'params',),
)

glGetArrayObjectivATI = platform.createExtensionFunction( 
	'glGetArrayObjectivATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLenum, arrays.GLintArray,),
	doc = 'glGetArrayObjectivATI( GLenum(array), GLenum(pname), GLintArray(params) ) -> None',
	argNames = ('array', 'pname', 'params',),
)

glVariantArrayObjectATI = platform.createExtensionFunction( 
	'glVariantArrayObjectATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, constants.GLsizei, constants.GLuint, constants.GLuint,),
	doc = 'glVariantArrayObjectATI( GLuint(id), GLenum(type), GLsizei(stride), GLuint(buffer), GLuint(offset) ) -> None',
	argNames = ('id', 'type', 'stride', 'buffer', 'offset',),
)

glGetVariantArrayObjectfvATI = platform.createExtensionFunction( 
	'glGetVariantArrayObjectfvATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, arrays.GLfloatArray,),
	doc = 'glGetVariantArrayObjectfvATI( GLuint(id), GLenum(pname), GLfloatArray(params) ) -> None',
	argNames = ('id', 'pname', 'params',),
)

glGetVariantArrayObjectivATI = platform.createExtensionFunction( 
	'glGetVariantArrayObjectivATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, arrays.GLintArray,),
	doc = 'glGetVariantArrayObjectivATI( GLuint(id), GLenum(pname), GLintArray(params) ) -> None',
	argNames = ('id', 'pname', 'params',),
)


def glInitVertexArrayObjectATI():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.framebuffer_object

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.framebuffer_object to provide a more 
Python-friendly API
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions, wrapper
from OpenGL.GL import glget
import ctypes
from OpenGL.raw.GL.EXT.framebuffer_object import *
### END AUTOGENERATED SECTION

glGenFramebuffersEXT = wrapper.wrapper(glGenFramebuffersEXT).setOutput(
                'framebuffers', 
                lambda x: (x,), 
                'n')
                
glGenRenderbuffersEXT = wrapper.wrapper(glGenRenderbuffersEXT).setOutput(
                'renderbuffers', 
                lambda x: (x,), 
                'n')

#glBindRenderbufferEXT # doesn't require wrapping
#glBindFramebufferEXT  # doesn't require wrapping
#glBindRenderbufferEXT # doesn't require wrapping
#glCheckFramebufferStatusEXT
#glDeleteFramebuffersEXT # should be wrapped to eliminate 'length'
#glDeleteRenderbuffersEXT # should be wrapped to eliminate 'length'
#glFramebufferRenderbufferEXT
#glFramebufferTexture1DEXT
#glFramebufferTexture2DEXT
#glFramebufferTexture3DEXT
#glGenFramebuffersEXT  # wrapped
#glGenRenderbuffersEXT # wrapped
#glGenerateMipmapEXT
#glGetFramebufferAttachmentParameterivEXT
#glGetRenderbufferParameterivEXT
#glInitFramebufferObjectEXT
#glIsFramebufferEXT
#glIsRenderbufferEXT
#glRenderbufferStorageEXT # doesn't require wrapping                                          
#glget.addGLGetConstant( GL_MAX_COLOR_ATTACHMENTS_EXT, (1,))
#glget.addGLGetConstant( GL_FRAMEBUFFER_BINDING_EXT, (1,))
#glget.addGLGetConstant( GL_RENDERBUFFER_BINDING_EXT, (1,))
#glget.addGLGetConstant( GL_MAX_RENDERBUFFER_SIZE_EXT, (1,))



from OpenGL.GL import *
from OpenGL.GLUT import *
from OpenGL.GLU import *
import time

resX,resY = (400,300 )

def display( ):
	glutSetWindow(window);
	glClearColor (0.0, 0.0, (time.time()%1.0)/1.0, 0.0)
	glClear (GL_COLOR_BUFFER_BIT)
	glFlush ()
	glutSwapBuffers()


if __name__ == "__main__":
	glutInit([])
	glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH)

	glutInitWindowSize(resX, resY)

	glutInitWindowPosition(0, 0)
	window = glutCreateWindow("hello")
	glutDisplayFunc(display)
	glutMainLoop()
	



"""Fix silly lack-of-API problems in logging module

Adds constants to the log objects.
Adds getException(err) to log objects to retrieve 
formatted exception or err if traceback not available.
"""
try:
	from cStringIO import StringIO
except ImportError, err:
	from StringIO import StringIO
import traceback, logging

getLog = logging.getLogger
from OpenGL import ERROR_LOGGING, FULL_LOGGING

def getException(error):
	"""Get formatted traceback from exception"""
	exception = str(error)
	file = StringIO()
	try:
		traceback.print_exc( limit=10, file = file )
		exception = file.getvalue()
	finally:
		file.close()
	return exception
logging.Logger.getException = staticmethod( getException )
logging.Logger.err = logging.Logger.error
logging.Logger.DEBUG = logging.DEBUG 
logging.Logger.WARN = logging.WARN 
logging.Logger.INFO = logging.INFO 
logging.Logger.ERR = logging.Logger.ERROR = logging.ERROR

if FULL_LOGGING:
	getLog( 'OpenGL.calltrace' ).setLevel( logging.INFO )

class _LoggedFunction( object ):
	"""Proxy that overrides __call__ to log arguments"""
	def __init__( self, base, log ):
		self.__dict__[''] = base 
		self.__dict__['log'] = log
	def __setattr__( self, key, value ):
		if key != '':
			setattr( self.__dict__[''], key, value )
		else:
			self.__dict__[''] = value 
	def __getattr__( self, key ):
		if key == '':
			return self.__dict__['']
		else:
			return getattr( self.__dict__[''], key )
class _FullLoggedFunction( _LoggedFunction ):
	"""Fully-logged function wrapper (logs all call params to OpenGL.calltrace)"""
	_callTrace = getLog( 'OpenGL.calltrace' )
	def __call__( self, *args, **named ):
		argRepr = []
		function = getattr( self, '' )
		for arg in args:
			argRepr.append( repr(arg) )
		for key,value in named.items():
			argRepr.append( '%s = %s'%( key,repr(value)) )
		argRepr = ",".join( argRepr )
		self._callTrace.info( '%s( %s )', function.__name__, argRepr )
		try:
			return function( *args, **named )
		except Exception, err:
			self.log.warn(
				"""Failure on %s: %s""", function.__name__, self.log.getException( err )
			)
			raise
class _ErrorLoggedFunction ( _LoggedFunction ):
	"""On-error-logged function wrapper"""
	def __call__( self, *args, **named ):
		function = getattr( self, '' )
		try:
			return function( *args, **named )
		except Exception, err:
			self.log.warn(
				"""Failure on %s: %s""", function.__name__, self.log.getException( err )
			)
			raise
	

def logOnFail( function, log ):
	"""Produce possible log-wrapped version of function

	function -- callable object to be wrapped
	log -- the log to which to log information
	
	Uses ERROR_LOGGING and FULL_LOGGING
	to determine whether/how to wrap the function.
	"""
	if ERROR_LOGGING or FULL_LOGGING:
		if FULL_LOGGING:
			loggedFunction = _FullLoggedFunction( function, log )
		else:
			loggedFunction = _ErrorLoggedFunction( function, log )
		return loggedFunction
	else:
		return function
